;------------------------------------------------------------------------------
; File Name   :	k6311_12 (690025.2).rom
; Base Address:	0000h Range: B800h - C000h Loaded length: 0800h
;
; Druckermodul 690025.2 f. K6311/12 (bis 3. Quartal 1987), ROM M013B8
;
; reass: Volker Pohlers 18.08.2008, letzte nderung: 24.03.2020
;------------------------------------------------------------------------------

		cpu	Z80
		page	0

;------------------------------------------------------------------------------
; OS
;------------------------------------------------------------------------------

; RAM
IOBYT		equ	0004h		; I/O-Byte zur Zuweisung log.Gert phy.	Gert
LISW		equ	0015h		; Schalter fr Hardcopy. 0 kein Copy, 1 Copy
KEYBU		equ	0025h		; Tastaturpuffer
EOR		equ	0036h		; Zeiger auf letzte fr	Anwender freie Adresse
SYSR		equ	02C0h		; reserviert fr Systemerweiterung
ATTYL		equ	0EFE1h		; Adresse TTY-Treiber fr LIST
TXLST		equ	0EFEFh		; Adresse einer	Zeichenkette des aktuellen LIST-Gertes
BWS		equ	0EC00h		; Bildwiederholspeicher

; CALL5
PRNST		equ 	09h		; Ausgabe einer	Zeichenkette zu	CONST
RCONB		equ 	0Ah		; Eingabe einer	Zeichenkette von CONST


;------------------------------------------------------------------------------
; Druckermodul
;------------------------------------------------------------------------------

SIODA		EQU	0B0H		; Datenadr SIO Kanal A
SIOKD		EQU	SIODA+2		; Kdoadr   - " -
CTC		EQU	0A8H		; Adr CTC Kanal 0

; Drucker mit Schnittstelle RS232C und DTR-Protokoll (Hardware-Protokoll)

; Baudrate:
;	BDRAT in Bd (50, 75, 100, 150, 200, 300, 600, 1200, 2400, 4800 oder 9600)
; 	Zeitkonstante fuer CTC: ZKCTC=9600/BDRAT
; Datenbit: DABI
;	7 Datenbit: DABI=20H
;	8 Datenbit: DABI=60H
; Paritaetsbit: PABI
;	ohne     Paritaet: PABI=2
;	gerade   Paritaet: PABI=3
;	ungerade Paritaet: PABI=1
; Stopbit: STPBI
;	1   Stopbit: STPBI=4
;	1.5 Stopbit: STPBI=8
;	2   Stopbit: STPBI=0CH

; K6311/12:
ZKCTC		EQU	1		; 9600 Bd
DABI		EQU	60H		; 8 Datenbit
PABI		EQU	2		; ohne Paritaet
STPBI		EQU	4		; 1 Stopbit

lnlen		equ	132		;max. Anzahl Zeichen in Zeilenpuffer
lnlen1		equ	30		;zustzlicher Puffer

;------------------------------------------------------------------------------
; OS-Rahmen
;------------------------------------------------------------------------------

		org 0B800h

		jp	eos
		db "#       ",0

		jp	dini
name:		db "K6311/12",0
		db    0

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

eos:		call	dini
		jp	0F089h		; Sprung ins OS

;Aufbau RAM-Speicher
;Lnge 182 Byte
;IX=(2C4)=(EOR)
;
;IX+0
;IX+1,2 akt. Pos. im Zeilenpuffer
;IX+3,4 Beginn Zeilenpuffer
;IX+5,6 Ende Zeilenpuffer
;IX+7,8 Merkzelle fr Zeilenpufferende (z.B. bei Wagenrcklauf)
;IX+9   Merkzelle fr CRLF, FFh -> letzten Zeichen war CR, sonst 0
;IX+10  Endlospapier ('J'/'N') Bit2=0 -> 'J'
;IX+11  Zeilen pro Seite
;IX+12  akt. Anz. Zeilen bis Seitenende
;IX+13  Leerzeilen pro Seite
;IX+14  Zeichen pro Zeile
;IX+15  Eingabepuffer, max Lnge
;IX+16		aktuelle Lnge
;IX+17..20	1..4 Zeichen
;IX+20..181 Zeilenpuffer (162 Zeichen)

;------------------------------------------------------------------------------
; Initialisierung
;------------------------------------------------------------------------------

dini:		ld	hl, (TXLST)	; Zeichenkette f. LIST eintragen
		ld	de, name	; "K6311/12"
		and	a		; Cy = 0
		sbc	hl, de		; TXLST schon gesetzt?
		push	af
		ld	(TXLST), de	; Treibername eintragen
		push	de
		jr	z, ini1		; Druckertreiber schon init.
; Erst-Initialisierung
		ld	hl, (EOR)
		ld	de, lnlen+lnlen1+20	; bentigter Arbeitsspeicher
		and	a
		sbc	hl, de
		ld	(EOR), hl	; neues	log. RAM-Ende
		ld	(2C4h),	hl	; merken
		push	hl
		pop	ix		; HL=IX+0=EOR, Arbeitsspeicher ab IX+1
		ld	de, 20		; 19 Byte Arbeitszellen reservieren
		add	hl, de
		ld	(ix+4),	h	; Beginn Zeilenpuffer
		ld	(ix+3),	l
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ld	de, lnlen	; Lnge	Zeilenpuffer
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l

		ld	(ix+9),	0	; CR-Merker rcksetzen
;Programmierung CTC und SIO
		ld	hl, PROGT
		ld	c, CTC		; Prog. CTC
		ld	b, 2
		otir
		ld	c, SIOKD	; Prog. SIO
		ld	b, 8
		otir
;
		call	clrzp		; Zeilenpuffer mit Leerzeichen init.
; Initialisierung bei WBOOT
ini1:		ld	hl, ttyl	; TTY-Treiber f. LIST
		ld	(ATTYL), hl
		ld	hl, 4		; ASGN LIST := TTYL
		res	7, (hl)
		res	6, (hl)
		xor	a
		ld	(LISW), a	; kein Hardcopy
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		pop	de		; DE=Treibername
		call	5		; ausgeben
		ld	b, 1
		call	stat1		; kurze	Statusabfrage
		ld	de, aNichtBereit ; " nicht bereit ! "
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	c, 5
		pop	af
		call	nz, inidlg	; wenn Drucker nicht init.
		and	a		; Cy = 0, d.h. kein Fehler
		ret


;------------------------------------------------------------------------------
; Initialisierungstabelle CTC und SIO
;------------------------------------------------------------------------------

PROGT:
;CTC
		db 00010111b		; Zeitgeber/DI/Vorteiler 16
		db ZKCTC		; Zeitkonstante
;SIO
		db 4			; Steuerregister
		db 40H+PABI+STPBI	; keine	Paritt,1 Stoppbit,Vorteiler 16,Monosynch-Mode
		db 1			; Interruptvektor
		db 0			; Kein Interrupt
		db 3			; Steuerung Empfnger
		db 11000000b		; 8bit,	Empf. aus
		db 5			; Steuerung Sender
		db DABI+8		; 8 bit, Sender	ein

aNichtBereit:	db " nicht bereit ! ",0

;------------------------------------------------------------------------------
; Einrichten des logischen Treibers
; (Bedienerdialog fr Papierformat etc.)
;------------------------------------------------------------------------------

inidlg:		ld	de, aEndlospapier ; "\r\n\nEndlospapier ? (J/N): "
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	5
		push	ix
		pop	hl
		ld	de, 15
		add	hl, de
		ex	de, hl		; DE:=IX+15 Eingabepuffer
		ld	a, 2
		ld	(de), a		; Lnge	des Puffers 2 Zeichen
		ld	c, RCONB	; Eingabe einer	Zeichenkette von CONST
		call	5
		inc	de
		ld	a, (de)		; A=akt. Lnge Zeichenkette
		cp	1
		jr	nz, inidlg	; mehr als 1 Zeichen
		inc	de
		ld	a, (de)		; in A Zeichen
		cp	'J'
		jr	z, inidlg1	; Zeichen in IX+10 ablegen
		cp	'N'
		jr	nz, inidlg	; Zeichen <> J/N

inidlg1:	ld	(ix+10), a	; Zeichen in IX+10 ablegen
		call	getlp		; Abfrage "Textzeilen"
;
		bit	2, (ix+10)
		call	z, getelp	; Abfrage "Leerzeilen pro Seite", wenn Endlospapier
;
		call	getcl		; Abfrage "Zeichen pro Zeile"
		ret

aEndlospapier:	db 0Dh,0Ah
		db 0Ah
		db "Endlospapier ? (J/N): ",0


; Abfrage "Textzeilen"
getlp:		ld	de, aTextzeilenProS ; "\r\nTextzeilen pro Seite	(1-255): "
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	5
		call	getnum		; Eingabe einer	Zahl (dez)
					; out: DE Zahl
		jr	c, getlp	; bei Fehler
		ld	a, d
		and	a
		jr	nz, getlp	; Zahl >= 256
		ld	a, e
		cp	0
		jr	z, getlp	; Zahl = 0
		ld	(ix+11), e	; Anzahl eintragen
		ld	(ix+12), e	; --"--
		ret

aTextzeilenProS:db 0Dh,0Ah
		db "Textzeilen pro Seite (1-255): ",0

; Abfrage "Leerzeilen pro Seite"
getelp:		ld	de, aLeerzeilenProS ; "\r\nLeerzeilen pro Seite	(0-255): "
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	5
		call	getnum		; Eingabe einer	Zahl (dez)
					; out: DE Zahl
		jr	c, getelp	; bei Fehler
		ld	a, d
		and	a
		jr	nz, getelp	; Zahl >= 256
		ld	(ix+13), e	; Anzahl eintragen
		ret

aLeerzeilenProS:db 0Dh,0Ah
		db "Leerzeilen pro Seite (0-255): ",0

; Abfrage "Zeichen pro Zeile"
getcl:		ld	de, aMax_Zeilenlaen ; "\r\nMax.	Zeilenlaenge   (10-132): "
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	5
		call	getnum		; Eingabe einer	Zahl (dez)
					; out: DE Zahl
		jr	c, getcl	; bei Fehler
		ld	a, d
		and	a
		jr	nz, getcl	; Zahl >= 256
		ld	a, e
		cp	lnlen+1
		jr	nc, getcl	; Zahl >= 133
		cp	10
		jr	c, getcl	; Zahl < 10
		ld	(ix+14), e	; Anzahl eintragen
;
		ld	de, aPapierPosition ; "\r\nPapier positionieren	!\r\n"
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	5
		ret

aMax_Zeilenlaen:db 0Dh,0Ah
		;db "Max. Zeilenlaenge   (10-132): ",0
		outradix 10
		db "Max. Zeilenlaenge   (10-\{lnlen}): ",0
aPapierPosition:db 0Dh,0Ah
		db "Papier positionieren !",0Dh,0Ah,0


;------------------------------------------------------------------------------
; Eingabe einer	Zahl (dez)
; out: DE Zahl
;------------------------------------------------------------------------------

getnum:		push	ix
		pop	hl
		ld	de, 15
		add	hl, de
		ex	de, hl		; in DE	Adr. Eingabepuffer
		ld	a, 4		; Lnge	max. 4 Zeichen
		ld	(de), a
		ld	c, RCONB	; Eingabe einer	Zeichenkette von CONST
		call	5
		inc	de
		ld	a, (de)		; akt. Lnge Zeichenkette
		cp	0
		jr	z, getnumerr	; Lnge=0
		cp	4
		jr	nc, getnumerr	; Lnge	>= 4
		ld	b, a
		push	bc		; in B Stellenanzahl
getnum1:	inc	de
		ld	a, (de)		; A:=Ziffer
		cp	'0'
		jr	c, getnum2	; keine	Ziffer
		cp	'9'+1
		jr	nc, getnum2	; keine	Ziffer
		djnz	getnum1
		jp	a2num		; Umwandlung ASCII (dez)->Zahl

getnum2:	pop	de
getnumerr:	scf			; Fehler! Cy:=1
		ret

; Umwandlung ASCII (dez)->Zahl
a2num:		ex	de, hl
		ld	de, 0		; HL=Pufferende
		pop	bc		; in B Stellenanzahl
		push	ix
		ld	iy, fakttab	; Multiplikatortabelle
a2num1:		ld	c, (hl)		; in C Ascii-Ziffer letzte Stelle
a2num2:		ld	a, c
		cp	'0'
		jr	z, a2num4	; wenn = 0
		ld	a, e
		add	a, (iy+0)	; Faktor addieren (C mal)
		ld	e, a
		jr	nc, a2num3	; kein bertrag
		inc	d
a2num3:		dec	c
		jr	a2num2
;
a2num4:		dec	hl		; nchste Ziffer
		inc	iy		; nchster Faktor
		djnz	a2num1		; bis alle Stellen fertig
		and	a		; Cy :=	0
		pop	iy
		ret

; Multiplikatortabelle
fakttab:	db 1
		db 10
		db 100

;------------------------------------------------------------------------------
; TTY-Treiber f. LIST (logischer Treiber)
;
; Treibereingangsparameter:
; - A Kommando (siehe unten)
; - C Zeichen (bei Kommando AUSGABE)
;
; Treiberausgangsparameter:
; - CY Fehlerstatus
;
; Kommando
; 00 Abfrage Status
; 	Return:
; 	A Status
; 	0 kein Zeichen bei Eingabegert, nicht bereit bei Ausgabegert
; 	sonst Zeichen liegt an bei Eingabegert,
; 	(im installierten CRT-Treiber wird der Zeichencode bergeben)
; 02 Ausgabe Zeichen
; 	Eingang:
; 	C Zeichen
; (FF Initialisieren/Rcksetzen des Gertes)
;
; Es wird nur 00 ausgewertet, alle anderen codes werden ignoriert -> Zeichenausgabe
; FF ist nicht implementiert
;------------------------------------------------------------------------------

ttyl:		and	a		; TTY-Treiber f. LIST
		ld	b, 1
		jp	z, stat1	; wenn A=0 -> Statusabfrage
		push	ix
		ld	ix, (2C4h)	; 
		ld	hl, ttylret
		push	hl		; RETURN-Adresse
		ld	a, c		; A=auszugebendes Zeichen
		ld	hl, ttyltab1	; Tabelle der Steuerzeichen
		ld	bc, 7
		cpir			; Steuerzeichen	suchen
		jr	z, ttyl1	; wenn A Steuerzeichen
		dec	bc		; bc :=	-1, wenn kein Steuerz., so Funktion char_xx
;
ttyl1:		ld	hl, 6
		and	a
		sbc	hl, bc
		add	hl, hl
		ld	bc, ttyltab2	; Tabelle der Steuerzeichen-Funktionen
		add	hl, bc
		ld	c, (hl)
		inc	hl
		ld	h, (hl)
		ld	l, c
		jp	(hl)		; Steuerzeichenfunktion	starten
;
ttylret:	pop	ix
		ret	nc
		ld	a, 0
		ld	(LISW), a	; kein Hardcopy
		ret

; Tabelle der Steuerzeichen
ttyltab1:	db 0Dh
		db 0Ah
		db 0Ch
		db 9
		db 17h
		db 0Eh
		db 8
; Tabelle der Steuerzeichen-Funktionen
ttyltab2:	dw char_0d		; Wagenrcklauf CR
		dw char_0a		; Cursor down
		dw char_0c		; CRLF (CLS?)
		dw char_09		; Cursor right
		dw char_17		; CTRL/W Seitenvorschub	am Drucker
		dw char_0e		; CTRL/N Bildschirmkopie
		dw char_08		; Cursor left
		dw char_xx		; alle anderen Zeichen


;------------------------------------------------------------------------------
; Steuerzeichenfunktionen
;------------------------------------------------------------------------------

; Wagenrcklauf CR
char_0d:	ld	(ix+9),	0FFh	; CR merken
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		and	a		; Cy := 0
		ret

; Cursor down LF
char_0a:	bit	0, (ix+9)	; war CR?
		jr	nz, char_0a_1	; wenn ja
;
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
char_0a_1:	ld	(ix+9),	0	; CR-Merker rcksetzen
		call	outzp		; Zeilenpuffer ausgeben
		ret

; CLS -> CRLF
char_0c:	call	char_0d		; CR
		call	char_0a		; LF
		ret

; Cursor right
char_09:	ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	d, (ix+6)	; Ende Zeilenpuffer
		ld	e, (ix+5)
		and	a		; Cy := 0
		sbc	hl, de		; Zeilenende erreicht?
		ccf
		call	z, char_0c	; ja -> CRLF
		ld	(ix+9),	0	; CR-Merker rcksetzen
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		inc	hl
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ret

; CTRL/W Seitenvorschub	am Drucker
char_17:	bit	2, (ix+10)	; Endlospapier?
		jr	nz, char_17_1	; nein
;
		ld	a, (ix+13)	; Leerzeilen pro Seite
		and	a		; <> 0 ?
		jr	nz, char_17_1
;
		call	char_0c		; CRLF
		ret
;wenn Endlospapier
char_17_1:	ld	b, (ix+12)	; akt. Anz. Zeilen bis Seitenende
char_17_2:	push	bc
		call	char_0c		; CRLF
		pop	bc
		ret	c
		djnz	char_17_2
		ret

; sonstige Zeichen
char_xx:	cp	20h
		jr	nc, char_xx_1	; Steuerzeichen < 20h bergehen
		and	a		; Cy := 0
		ret
;
char_xx_1:	push	af
		call	char_09		; Cursor right
		pop	de
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(hl), d		; Zeichen im Zeilenpuffer eintragen
		ret


; CTRL/N Bildschirmkopie
char_0e:	call	char_0c		; CRLF
		ret	c
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 40
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 40 Zeichen/Zeile verkleinern
;
		ld	de, BWS		; Bildwiederholspeicher
char_0e_1:	ld	a, (de)
		push	de
		call	char_xx		; Zeichen ausgeben (logisch)
					; Zeilenwechsel erfolgt automatisch
		pop	de
		ret	c
		inc	de
		ld	hl, BWS+40*24-1	; BWS-Ende
		and	a
		sbc	hl, de
		jr	nc, char_0e_1
;
		call	char_0c		; CRLF
		ret	c
;
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 132
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 132 Zeichen/Zeile restaurieren
		ret

; Cursor left
char_08:	ld	(ix+9),	0	; CR-Merker rcksetzen
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	d, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	e, (ix+1)
		and	a
		sbc	hl, de		; Am Anfang der Zeile?
		ret	z		; dann gibt es nicht zu tun
		dec	de		; sonst 1 Zeichen nach links
		ld	(ix+2),	d	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	e
		and	a
		ret


;------------------------------------------------------------------------------
; Zeilenpuffer-Routinen
;------------------------------------------------------------------------------

; Zeilenpuffer mit Leerzeichen init.
clrzp:		push	af
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	bc, lnlen+lnlen1	; Lnge Zeilenpuffer
clrzp1:		inc	hl
		ld	(hl), ' '
		dec	bc
		ld	a, b
		or	c
		jr	nz, clrzp1
		pop	af
		ret

; Zeilenpuffer ausgeben
outzp:		call	prepzp		; Zeilenpuffer fr Ausdrucken vorbereiten
		call	prnzp		; Ausdrucken einer Zeile
		call	clrzp		; Zeilenpuffer mit Leerzeichen init.
		ret

; Zeilenpuffer fr Ausdrucken vorbereiten
prepzp:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		push	hl
		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
		and	a
		sbc	hl, bc
		ld	b, h		; BC := HL := Anzahl auszugebender Zeichen
		ld	c, l
		ex	(sp), hl	; HL := Zeilenpufferende
		push	hl
		jr	z, prepzp1	; wenn keine Zeichen auszugeben
; sonst Zeilenpuffer 1 Zeichen nach rechts verschieben
		ld	d, h
		ld	e, l
		inc	de
		lddr
;
prepzp1:	inc	hl
		ld	(hl), 0Dh	; als erstes Zeichen CR eintragen
;
		pop	hl		; HL := Zeilenpufferende
prepzp2:	inc	hl
		ex	(sp), hl
		ld	b, 0
		ld	c, (ix+14)	; Zeichen pro Zeile
		and	a
		sbc	hl, bc
		ld	b, h
		ld	c, l
		ex	(sp), hl
		jr	z, prepzp3
		jr	c, prepzp3
; wenn Zeilenlnge erreicht, so LFCR einfgen
		push	hl
		ld	d, h
		ld	e, l
		inc	de
		inc	de
		lddr
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		inc	hl
		ld	(hl), 0Dh	; CR einfgen
		pop	hl
		inc	hl
		jr	prepzp2
;
prepzp3:	pop	de
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		ld	(ix+8),	h	; Merkzelle fr Zeilenpufferende
		ld	(ix+7),	l
		ret

; Ausdrucken einer Zeile
prnzp:		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
prnzp1:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		and	a
		sbc	hl, bc
		ret	z		; wenn kein Zeichen auszugeben ist
;
prnzp2:		inc	bc
		ld	a, (bc)
		push	bc
		call	prpgr		; Druckzeichen aus A nach D kopieren,
					; Grafikzeichen	als '*'
		call	dout		; Ausgabe D
		pop	bc
		ret	c
		ld	a, (bc)
		cp	0Ah
		jr	nz, prnzp2	; Zeichen ausgeben bis LF
;
		call	newpg		; dann evtl. Seitenwechsel
		jr	prnzp1		; und restl. Zeichen ausgeben

; Seitenwechsel
newpg:		dec	(ix+12)		; akt. Anz. Zeilen bis Seitenende
		ret	nz
;
		ld	a, (ix+11)	; Zeilen pro Seite
		ld	(ix+12), a	; akt. Anz. Zeilen bis Seitenende
		bit	2, (ix+10)	; Endlospapier?
		jr	z, newpg2	; nein
;
		ld	hl, KEYBU	; Warten auf Tastendruck
newpg1:		ld	a, (hl)
		and	a
		jr	z, newpg1
		ld	(hl), 0
		ret
; Endlospapier --> Ausgabe der Leerzeilen
newpg2:		ld	e, (ix+13)	; Leerzeilen pro Seite
		ld	a, e
		and	a
		ret	z
newpg3:		ld	d, 0Ah
		push	bc
		call	dout		; Ausgabe D
		pop	bc
		jr	c, newpgret	; bei Fehler
		dec	e
		jr	nz, newpg3
		ret


; Druckzeichen aus A nach D kopieren,
; Grafikzeichen	als '*'
prpgr:		ld	d, a
		cp	7Fh
		ret	c
		ld	d, '*'
		ret

;
newpgret:	pop	bc
		ret

;------------------------------------------------------------------------------
; phys. Treiber
;------------------------------------------------------------------------------

; Ausgabe D
dout:		call	stat		; lange	Statusabfrage
		ret	c
		ld	a, d
		out	(SIODA), a
		and	a
		ret

; Statusabfrage
status:		and	a
		ld	a, 10h
		out	(SIOKD), a
		in	a, (SIOKD)	; SIO RR0
		bit	5, a
		ret	nz
		scf
		ret

; lange	Statusabfrage
stat:		ld	b, 200		; Abfrage Status

; kurze	Statusabfrage
stat1:		ld	c, 200
stat2:		call	wait		; Zeitschleife
		call	status		; Statusabfrage
		ret	nc
		dec	c
		jr	nz, stat2
		djnz	stat1		; kurze	Statusabfrage
		scf
		ret

; Zeitschleife
wait:		push	bc
		ld	bc, 391
wait1:		dec	c
		nop
		jr	nz, wait1
		djnz	wait1
		pop	bc
		ret


		end

; "restmll"

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

		sbc	a, (hl)
		cp	h
		jr	c, loc_BC9B
		ld	d, 2Fh
		ret

		cp	60h
		jr	nz, loc_BC95
		ld	d, 89h
		call	sub_BC9E
		jr	c, loc_BC9B
		ld	d, 20h
		ret

loc_BC95:	cp	7Fh
		ret	c
		ld	d, '*'
		ret

loc_BC9B:	pop	bc
		pop	bc
		ret

sub_BC9E:	call	sub_BCB3
		ret	c
		ld	a, d
		out	(SIODA), a
		and	a
		ret

sub_BCA7:	and	a
		ld	a, 10h
		out	(SIOKD), a
		in	a, (SIOKD)
		bit	5, a
		ret	nz
		scf
		ret

sub_BCB3:	ld	b, 200
loc_BCB5:	ld	c, 200
loc_BCB7:	call	sub_BCC5
		call	sub_BCA7
		ret	nc
		dec	c
		jr	nz, loc_BCB7
		djnz	loc_BCB5
		scf
		ret

sub_BCC5:	push	bc
		ld	bc, 187h
loc_BCC9:	dec	c
		nop
		jr	nz, loc_BCC9
		djnz	loc_BCC9
		pop	bc
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

		end
