; File Name   :	c:\user\hobby\stick\hobby\z9001	interfaces\TREIBER1.ROM
; Format      :	Binary file
; Base Address:	0000h Range: 0300h - 0538h Loaded length: 0238h

;------------------------------------------------------------------------------
; Treiberpaket
; Dr. Frank Schwarzenberg
; Zentralinstitut fr Kernforschung der AdW der DDR
; Rossendorf, Dresden
; reass: 06.06.2008 V. Pohlers
;------------------------------------------------------------------------------

		cpu	Z80

		include	z9001.inc

;System-RAM
;LISW		EQU	15H		; SCHALTER FUER DRUCKERAUSGABE
KEYBU		EQU	25H		; TASTATURPUFFER
WORKA		equ	33h 		; Hilfszelle
EOR		equ	36h		; Zeiger auf letzte fuer Anwender freie Adresse


;------------------------------------------------------------------------------
; BEEP: Erzeugt einen "sauberen" (im Gegensatz zu CTRL/Q) Tastatur-Quittungston,
; 	der die Eingabesicherheit der Tastatur verbessert.
; SIFE: SIF1000-Eingabe, 7 Datenbits parallel
; SIFA: SIF1000-Ausgabe, 7 Datenbits parallel
; CENTR: 7-Bit-CENTRONICS-Druckerinterface
; TD40: V24/DTR, 8 Bit, keine Paritt, 1200 Baud
; LX86: V24/DTR, 8 Bit, keine Paritt, 9600 Baud
; 
; Das Programm wird normal (nicht mit ASGN) geladen. Es enthlt einen Mechanismus, 
; der das eigentliche Treiberpaket an das aktuelle RAM-Ende verschiebt und eine 
; entsprechende Adremodifikation vornimmt. Das Programm ist damit fr jede RAM-
; Konfiguration geeignet und mu nicht speziell angepat werden. Die aktuelle 
; obere Speichergrenze wird fr BASIC-Anwender dezimal ausgegeben. Dieser Wert 
; sollte nach Start des BASIC-Interpreters als obere Grenze eingegeben werden, um 
; das Treiberpaket vor berschreiben zu schtzen.
; 
; Die gewnschten Treiber *mssen* vorher mit der ASGN-Anweisung aktiviert werden. 
; Dabei sind die folgenden Zuordnungen mglich:
; 
; 		physische Gertenummer 
; log. Gert	0	1	2	3
; --------------------------------------------
; CONST:  (0)	-	(CRT) 	(BAT) 	BEEP
; READER: (2) 	-	-	SIFE	-
; PUNCH:  (4)	-	CENTR 	LX86 	SIFA 
; 				TD40	
; LIST:	  (6)	-	CENTR 	LX86 	SIFA 
; 				TD40	
;------------------------------------------------------------------------------

		org 	300h

beg:		jp	SIFE
		db 	"SIFE    ",0
		jp	SIFA
		db 	"SIFA    ",0
		jp	CENTR
		db 	"CENTR   ",0
		jp	LX86
		db 	"LX86    ",0
		jp	TD40
		db 	"TD40    ",0
		jp	BEEP
		db 	"BEEP    ",0
		db    0

;------------------------------------------------------------------------------
; Inititialisierung der Treiber
; das *muss* ber ASGN log:=phys erfolgen!
;------------------------------------------------------------------------------

BEEP:		ld	bc, BEEP1
		ld	hl, 3		; log. Gertenr.=0, phys. Gertenr.=3
		jr	INIT4

SIFE:		ld	bc, SIFE1
		ld	hl, 202h	; log. Gertenr.=2, phys. Gertenr.=2
		ld	d, 7Fh		; PIO1B-Initialisierung
		jr	INIT3

SIFA:		ld	bc, SIFA1
		ld	l, 3		; phys. Gertenr.=3
		ld	d, 0		; PIO1B-Initialisierung
		jr	INIT2

CENTR:		ld	bc, CNTR1
		ld	l, 1		; phys. Gertenr.=1
		ld	d, 0		; PIO1B-Initialisierung
		jr	INIT2

LX86:		ld	bc, V24_1
		ld	a, 12		; TC=12 ==> 9600 baud
		jr	INIT1

TD40:		ld	bc, V24_1
		ld	a, 126		; TC=126 ==> 1200 baud

INIT1:		ld	(V24_4+1), a	; TC patchen
		ld	l, 2		; phys. Gertenr.=2
		ld	d, 0FEh		; PIO1B-Initialisierung
;
INIT2:		ld	a, (WORKA)	; Zwischenergebnis von ALDEV
		ld	h, a
		sub	4		; Test auf log. Gertenr.
		ld	a, 1		; Fehler,
		ret	c		; wenn nicht PUNCH (4) oder LIST (6)
;
INIT3:		ld	a, 0CFh
		out	(PIO1BS), a	; PIO1 B Kommando Anwenderport
		ld	a, d
		out	(PIO1BS), a	; PIO1 B Kommando Anwenderport
		cpl
		out	(PIO1BD), a
; nach CPROM enthlt DE die Adresse des Kommandos im Speicher
; hier wurde D verndert, deshalb restaurieren
		ld	d, hi(beg)

;bergabe an ASGN:
;H = logische Gertenummer
;L = physische Gertenummer
;BC = Treiberadresse
;DE = Adr. des symb. Namens

INIT4:		or	a		; CY=0,	kein Fehler
		ex	(sp), hl	; dieser Teil
		inc	hl		; ist nur bei OS Version 1.1
		inc	hl		; noetig (wegen	eines Fehlers
		ex	(sp), hl	; in der ASGN-Routine)
		ret

;------------------------------------------------------------------------------
; Test auf Stop-Taste gedrckt?
;------------------------------------------------------------------------------

STPRQ:		call	0FD33h		; DECO0
		ei
		sub	3		; STOP-Taste?
		or	a		; wegen	CY
		ret	nz		; nein
		ld	(KEYBU), a	; loeschen
		scf			; STOP-Taste anzeigen
		ret

;------------------------------------------------------------------------------
; V24-Ausgabe ueber User-Port
;
;- DTR-Prozedur
;- Beispielprogremm fuer serielle Ausgabe ueber
;  einen PIO-Port
;- Einstellbare Parameter:
;  . Anzahl der Datenbits: DN (max.8)
;  . Baudrate: 19200,9600,4800,2400,1200,600
;    TC =          4   12   28   60  126 255
;
;- Serielle Ausgabe ueber PIO-Bit 0
;- Bereitschaftsabfrage (DTR) ueber PIO-Bit 7
;- Zeichen in C
;------------------------------------------------------------------------------
;
;	User-Port-Belegung V24-Ausgabe
;
;		A	B	C
;	---------------------------
;	1	0	0	-
;	2	TxD	-	-
;	3	-	-	-
;	4	-	DTR*	-
;	5	(+5V)	-	-
;
;	* DTR-Signal des Druckers
;------------------------------------------------------------------------------

V24_1:		inc	a		; A=FF (=Treiberinitialisierung) ?
		ret	z		; ja
		dec	a		; A=0 (=Abfrage Status)?
		jr	nz, V24_2	; nein -> Ausgabe
; Statusabfrage
		in	a, (PIO1BD)	; DTR abfragen
		or	7Fh
		cpl
		ret
; Zeichenausgabe
V24_2:		call	STPRQ		; Stop-Taste?
		jr	nc, V24_3
		ld	(LISW), a	;LIST: abschalten
		ret
;
V24_3:		in	a, (PIO1BD)
		add	a, a		;Drucker bereit? 
		jr	c, V24_2	;nein, warten
		ld	a, c		;Zeichen nach A
;Parallel-Serien-Wandlung und Ausgabe
V24_4:		ld	e, 7Eh		;Zeitkonstante TC (wird bei INIT gepatcht)
		ld	b, 9		;1 Startbit, 8-Datenbit
		di			;keine Unterbrechung zulassen
		or	a		;CY=0
		rla			;Start-Bit
V24_5:		out	(PIO1BD), a	;Bit ausgeben
		call	V24_6		;eine Bit-Zeit warten
		rra			;naechetes Bit 
		djnz	V24_5
		or	1		;Stopbit
		out	(PIO1BD), a	
		ei			;Unterbrechung. wieder zulassen
;eine Bit-Zeit warten
V24_6:		push	de
V24_7:		dec	e
		jr	nz, V24_7
		pop	de
		ret

;------------------------------------------------------------------------------
; CENTRONICS-Druckerschnittstelle ueber User-Port
;
;- 7 Datenbits verfuegbar
;- PIO-Bit7 wird zur Bildung des CBNTRONICS-/STROBE-
;  Signals verwendet.
;- /ACKNLG vom Drucker wird ueber den CTC-Kanal erfasst.
;
;- Zeichen in Reg. C
;- Return: Carry=1 bei Stop-Taste
;------------------------------------------------------------------------------
;
;User-Port-Belegung fuer CENTRONICS-Anschluss
;
;		A	B	C
;	---------------------------
;	1 	0 	0	/ACC
;	2	DAT1	DAT2	DAT3
;	3	DAT4	DAT5	DAT6
;	4	DAT7 	/STROBE	-
;	5	(+5V)==> +5V	-
;
;------------------------------------------------------------------------------

CNTR1:		inc	a		; A=FF (=Treiberinitialisierung) ?
		ret	z		; ja
		dec	a		; A=0 (=Abfrage Status)?
		cpl
		ret	z		; Rckgabe A=FFH bei Abfrage Status
;
		ld	a, 57h		;Zaehler,pos. Planke
		out	(CTC1),	a
		out	(CTC1),	a	;INIT CTC-1
		ld	a, c
		or	80h		;/STROBE=HIGH
		cp	0FFh
		jr	nz, CNTR2
		nop
		nop			;kurz Warten
CNTR2:		out	(PIO1BD), a	;Zeichen ohne Strobe
		and	7Fh		;/STROBE=LOW
		out	(PIO1BD), a	;Daten gueltig
		or	80h		;/STROBE=HIGH
		out	(PIO1BD), a
CNTR3:		call	STPRQ		; Stop-Taste?
		ret	c
		in	a, (CTC1)
		cp	57h		;/ACKNLG?
		jr	z, CNTR3	;nein, warten
		xor	a
		ret


;------------------------------------------------------------------------------
;  SIF-1000 Ein-/Ausgabe ueber User-Port
;  -------------------------------------
;  Voraussetzungen
;- RUF wird ueber PIO-Bit 7 gebildet ==> nur 7 Datenbits
;  und keine Paritaet moeglich!
;- Die Flanken von END-A werden mit dem CTC-KANAL erfasst
;- keine KOM-Ausgabe
;- keine STAT-Auswertung
;
;  Ausgabe eines Zeichens an SIF-A
;  -------------------------------
;- Zeichen im C-Register
;- Return: Carry=1 ==> Stop-Taste
;------------------------------------------------------------------------------
;
;User-Port-Belegung fuer SIF-1000-Anschluss
;
;		A	B	C
;	---------------------------
;	1 	0 	0	/END
;	2	/DAT1	/DAT2	/DAT3
;	3	/DAT4 	/DAT5 	/DATE
;	4	/DAT7 	/RUF	-
;	5	(+5V)--> +5V	-
;
;------------------------------------------------------------------------------

SIFA1:		ld	a, 47h		;DI,Zaehler,neg.Flanke,TC,RESET
		out	(CTC1),	a	;INIT CTC-1
		out	(CTC1),	a	;TC=47H (willkuerlich)
		ld	a, c
		cpl			;Bei SIF-1000 Daten low active
		set	7, a		;ohne RUF-A
		out	(PIO1BD), a	;Zeichen ausgeben
		nop
		nop			;Einschwingen lassen
		res	7, a		;RUF-A
		out	(PIO1BD), a	;Zeichen gueltig
SIFA2:		call	STPRQ		; Stop-Taste?
		jr	nc, SIFA4	;nein
SIFA3:		cpl
		out	(89h), a	;RUF-A zuruecknehmen
		ret
SIFA4:		in	a, (CTC1)
		cp	47h		;END-A?
		jr	z, SIFA2	;nein,warten
		ld	a, 57h		;s.o.,aber pos. Flanke
		out	(CTC1),	a
		out	(CTC1),	a	;TC=57H
		ld	a, 0FFh		;/RUF-A
		out	(PIO1BD), a
SIFA5:		call	STPRQ		; Stop-Taste?
		jr	c, SIFA3	;ja
		in	a, (CTC1)
		cp	57h		;/END-A?
		jr	z, SIFA5	;nein, warten
		or	a
		ret


;------------------------------------------------------------------------------
;  Eingabe eines Zeichens von SIF-E
;  --------------------------------
;- Return: Zeichen in A
;- Carry=1 ==> STOP-Taste
;------------------------------------------------------------------------------

SIFE1:		ld	a, 47h		; Zaehler, neg.Flanke
		out	(CTC1),	a
		out	(CTC1),	a	; TC
		out	(PIO1BD), a	; RUF-E ausgeben
SIFE2:		call	STPRQ		; Stop-Taste?
		jr	c, SIFA3	; ja
		in	a, (CTC1)
		cp	47h		; END-E?
		jr	z, SIFE2	; NEIN
		in	a, (PIO1BD)	; DATEN
		cpl
		out	(PIO1BD), a	; /RUF-E
		and	7Fh
		ret

;------------------------------------------------------------------------------
; BEEP: Erzeugt einen "sauberen" (im Gegensatz zu CTRL/Q) Tastatur-Quittungston,
; 	der die Eingabesicherheit der Tastatur verbessert.
;------------------------------------------------------------------------------

BEEP1:		call	0F8F1h
		push	af
		ld	a, (2Fh)
		dec	a
		call	z, BEEP2
		pop	af
		ret
BEEP2:		push	bc
		ld	b, 0
		ld	c, 14h
		ld	a, 7
		out	(CTC0),	a	; System CTC0 Kassette,	Beeper
		ld	a, 96h
		out	(CTC0),	a	; System CTC0 Kassette,	Beeper
		in	a, (PIO1AD)	; PIO1 A Daten Beeper, Border, 20Z
		set	7, a
		out	(PIO1AD), a	; PIO1 A Daten Beeper, Border, 20Z
BEEP3:		djnz	BEEP3
		dec	c
		jr	nz, BEEP3
		res	7, a
		out	(PIO1AD), a	; PIO1 A Daten Beeper, Border, 20Z
		ld	a, 3
		out	(CTC0),	a	; System CTC0 Kassette,	Beeper
		pop	bc
		ret

;------------------------------------------------------------------------------
; Initialisierung des Treiberpakets
; Verschieben des Treibers ans RAM-Ende
; dieser Teil wird nicht mit kopiert!
;------------------------------------------------------------------------------

START:		ld	c, 9
		ld	de, aTopOfRam	; "\r\nTop of RAM now (dec.): "
		call	5
		ld	hl, (EOR)
		ld	bc, START-beg	; Lnge des zu kopierenden Bereichs
		or	a
		sbc	hl, bc		; EOR verringern
		ld	l, 0		; volle xx00h-Adresse
;
		push	hl
		dec	hl
		ld	(EOR), hl	; EOR setzen
; EOR (HL) dezimal anzeigen
		push	bc
		push	hl
		ld	b, 10h
		ld	de, 0
		ld	c, e
START1:		add	hl, hl
		ld	a, e
		adc	a, e
		daa
		ld	e, a
		ld	a, d
		adc	a, d
		daa
		ld	d, a
		ld	a, c
		adc	a, c
		ld	c, a
		djnz	START1
		ld	a, c
		or	a
		ex	de, hl
		jr	z, START2
		call	outa		; Anzeige A hexadezimal
START2:		call	outhl		; Anzeige HL hexadezimal
		ld	de, aCRLF	; "\r\n"
		ld	c, 9
		call	5
		pop	hl		; HL=EOR
		pop	bc		; BC=Lnge des zu kopierenden Bereichs
; Treiberpaket verschieben
		inc	hl		; hinter EOR
		ex	de, hl
		ld	hl, beg		; Programmanfang
		ldir			; Programm verschieben
;
		pop	hl		; HL=EOR+1
		ld	ix, TABOFFS
		ld	d, hi(beg)
		ld	e, h
		ld	b, 0
START3:		ld	c, (ix+0)
		inc	c
		dec	c
		ret	z		; 0 = Tabellenende
		add	hl, bc
		ld	a, (hl)
		sub	d
		add	a, e
		ld	(hl), a
		inc	ix
		jr	START3

;------------------------------------------------------------------------------
; Hilfsroutinen
;------------------------------------------------------------------------------

; Anzeige HL hexadezimal
outhl:		ld	a, h
		call	outa		; Anzeige A hexadezimal
		ld	a, l

; Anzeige A hexadezimal
outa:		push	af
		rrca
		rrca
		rrca
		rrca
		call	outa1
		pop	af
outa1:		and	0Fh
		add	a, 30h 		; '0'
		ld	c, 2
		ld	e, a
		jp	5

;------------------------------------------------------------------------------
; Texte
;------------------------------------------------------------------------------

aTopOfRam:	db	0Dh,0Ah
		db	"Top of RAM now (dec.): ",0
aCRLF:		db	0Dh,0Ah,0


;------------------------------------------------------------------------------
; Tabelle der Adressoffsets zur Adresskorrektur
;------------------------------------------------------------------------------
; vp: bei nderung im Code muss die Tabelle neu zusammengestellt werden
;     das kann mit genoffstab.pl erfolgen:
;     2x auf verschieden ORG-Adressen erstellen, z.B 300h und 400h	
;     Lnge des zu verschiebenden Bereichs ist movlength = 188h Byte
;     genoffstab.pl treiber1_0300.bin treiber1_0400.bin 0188 > offstab.inc
;------------------------------------------------------------------------------

movlength	equ 	START-BEG
TABOFFS:
		include offstab.inc

;orig. Tabelle		
;		db    2
;		db  0Ch
;		db  0Ch
;		db  0Ch
;		db  0Ch
;		db  0Ch
;		db  0Dh
;		db    8
;		db  0Ah
;		db    9
;		db    9
;		db    7
;		db    5
;		db  19h
;		db  21h
;		db  18h
;		db  2Fh
;		db  1Eh
;		db  19h
;		db  15h
;		db  1Bh
;		db    0

		end
